@@ -26,6 +26,7 @@ Created: Wed 22 Apr 2020 06:31:30 AM EDT
 """
 import os
 import logging
+import math
 from tempfile import TemporaryDirectory
 from pathlib import Path
 from timeit import default_timer as timer
@@ -37,6 +38,7 @@ from elastic_blast.aws import ElasticBlastAws
 from elastic_blast.aws import check_cluster as aws_check_cluster
 from elastic_blast.filehelper import open_for_read, open_for_read_iter, copy_to_bucket
 from elastic_blast.filehelper import check_for_read, check_dir_for_write, cleanup_temp_bucket_dirs
+from elastic_blast.filehelper import get_length
 from elastic_blast.split import FASTAReader
 from elastic_blast.jobs import read_job_template, write_job_files
 from elastic_blast.gcp import get_gke_credentials, delete_cluster_with_cleanup
@@ -60,16 +62,18 @@ from elastic_blast.taxonomy import setup_taxid_filtering
 from elastic_blast.config import validate_cloud_storage_object_uri
 from elastic_blast.elb_config import ElasticBlastConfig
 
+
 # TODO: use cfg only when args.wait, args.sync, and args.run_label are replicated in cfg
 def submit(args, cfg, clean_up_stack):
     """ Entry point to submit an ElasticBLAST search
     """
     dry_run = cfg.cluster.dry_run
-    cfg.validate(ElbCommand.SUBMIT)
+    cfg.validate(ElbCommand.SUBMIT, dry_run)
 
     # For now, checking resources is only implemented for AWS
     if cfg.cloud_provider.cloud == CSP.AWS:
-        check_resource_quotas(cfg)
+        if not dry_run:
+            check_resource_quotas(cfg)
     else:
         enable_gcp_api(cfg)
     
@@ -88,9 +92,26 @@ def submit(args, cfg, clean_up_stack):
     #mode_str = "synchronous" if args.sync else "asynchronous"
     #logging.info(f'Running ElasticBLAST on {cfg.cloud_provider.cloud.name} in {mode_str} mode')
 
-    # split FASTA query into batches
-    clean_up_stack.append(cleanup_temp_bucket_dirs)
-    queries, query_length = split_query(query_files, cfg)
+    cluster_name = cfg.cluster.name
+
+    use_1_stage_cloud_split = 'ELB_USE_1_STAGE_CLOUD_SPLIT' in os.environ
+    # Case for cloud split on AWS: one file on S3
+    cloud_split = False
+    if use_1_stage_cloud_split and cfg.cloud_provider.cloud == CSP.AWS \
+      and len(query_files) == 1 and query_files[0].startswith('s3://'):
+        cloud_split = True
+        query_file = query_files[0]
+        # Get file length as approximation of sequence length
+        query_length = get_length(query_file)
+        if query_file.endswith('.gz'):
+            query_length = query_length * 4 # approximation again
+        batch_len = cfg.blast.batch_len
+        nbatch = math.ceil(query_length/batch_len)
+        queries = nbatch * [query_file]
+    else:
+        # split FASTA query into batches
+        clean_up_stack.append(cleanup_temp_bucket_dirs)
+        queries, query_length = split_query(query_files, cfg)
 
     # setup taxonomy filtering, if requested
     setup_taxid_filtering(cfg)
@@ -100,7 +121,7 @@ def submit(args, cfg, clean_up_stack):
         elastic_blast = ElasticBlastAws(cfg, create=True)
         upload_split_query_to_bucket(cfg, clean_up_stack, dry_run)
         elastic_blast.upload_query_length(query_length)
-        elastic_blast.submit(queries)
+        elastic_blast.submit(queries, cloud_split)
         return 0
 
 
@@ -211,6 +232,8 @@ def check_running_cluster(cfg: ElasticBlastConfig) -> bool:
                  false if neither cluster is running nor results
                  are present
     """
+    if cfg.cluster.dry_run:
+        return False
     metadata_dir = os.path.join(cfg.cluster.results, ELB_METADATA_DIR)
     if cfg.cloud_provider.cloud == CSP.AWS:
         metadata_file = os.path.join(metadata_dir, ELB_AWS_JOB_IDS)
